@@ -250,7 +250,7 @@ static void
 goc_canvas_dispose (GObject *obj)
 {
 	GocCanvas *canvas = GOC_CANVAS (obj);
-	goc_group_clear (canvas->root);
+	g_object_run_dispose (G_OBJECT (canvas->root));
 	parent_klass->dispose (obj);
 }
 
@@ -290,6 +290,7 @@ goc_canvas_init (GocCanvas *canvas)
 	                       GDK_KEY_PRESS_MASK |
 	                       GDK_KEY_RELEASE_MASK |
 	                       GDK_ENTER_NOTIFY_MASK |
+	                       GDK_LEAVE_NOTIFY_MASK |
 #ifdef HAVE_GDK_EVENT_GET_SCROLL_DELTAS
 	                       GDK_SMOOTH_SCROLL_MASK |
 #endif
